/*
 * Copyright (C) 2018 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
 *
 * This file is part of pinsis
 *
 * pinsis is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
*/

import { Component, ViewChild, ElementRef, AfterViewInit, Inject } from '@angular/core';
import { chart } from 'highcharts';
import { Highcharts } from 'angular-highcharts';

import { GraphElementComponent } from './graph-element/graph-element.component';

import { graphSeries, Report, ReportService } from './../report.service';
import { Observable } from 'rxjs/Observable';

@Component({
  selector: 'graphs-page',
  templateUrl: './graphs-page.component.html',
  styleUrls: ['./graphs-page.scss']
})



export class GraphsPageComponent implements AfterViewInit {
  @ViewChild(GraphElementComponent)
  private graphComponent: GraphElementComponent;

  collectionSeries: CollectionGraphSeries;

  graphOptions: any;
  selectedGraphType: string;
  selectedStatusType: string;
  selectedInterval: string;
  intervalList: Array<string>;
  startDate: Date;
  endDate: Date;
  graphStartDate: Date;

  constructor(@Inject('ReportService') private reportService: ReportService) {
    this.collectionSeries = new CollectionGraphSeries();

    this.intervalList = ["Mensal", "Diário"];
    this.selectedInterval = this.intervalList[1];

    this.endDate = new Date();
    this.startDate = new Date();
    this.startDate.setDate(this.endDate.getDate() - 7);
    this.graphStartDate = new Date();

    this.setChartOptions("line");
    this.selectedStatusType = "all"
    this.updateChart();
  }

  ngAfterViewInit() {
  }
  private setChartOptions(graphType: string) {
    this.selectedGraphType = graphType;
    /* We need to reset the options to make sure previous graphs don't mess with the current */
    this.graphOptions = {};

    /* Common options */
    this.graphOptions.colors = ['#8e8cc4', '#4f425b', '#c5dcf2'];
    this.graphOptions.subtitle = { text: "Unidade monitorada: Hospital Erasto Gaertner" };
    this.graphOptions.credits = { enabled: false };

    /* Populate graph based on its type */
    this.graphOptions.series = [];

    if (this.selectedGraphType == "line") {


      this.graphStartDate = new Date(this.collectionSeries.start_date);
      this.graphStartDate.setDate(this.graphStartDate.getDate() + 1);

      let interval: number = 24 * 3600 * 1000; // Graph tick interval in milliseconds
      this.graphOptions.chart = { type: 'column' };
      if (this.selectedInterval == "Mensal" || this.selectedInterval == "Anual")
        interval *= 30;
      if (this.selectedInterval == "Anual")
        interval *= 12;

      this.graphOptions.title = { text: "Distribuição dos atendimentos ao longo do tempo" };
      this.graphOptions.yAxis = { 
        title: { text: "% de pacientes" },
        tickInterval: 25,
        max: 100
      };
      // Cast from String to Date
      this.startDate = new Date(this.startDate);
      this.endDate = new Date(this.endDate);
      this.graphOptions.xAxis = {
        title: {
          text: "Série temporal de " + String(this.startDate.getDate()) + "/" + String(this.startDate.getMonth()+1)
          + "/" + String(this.startDate.getFullYear()) + " a " +  String(this.endDate.getDate()) + "/" + String(this.endDate.getMonth()+1)
          + "/" + String(this.endDate.getFullYear())
        },
        type: "datetime",
        tickInterval: interval
      };


      this.graphOptions.tooltip = { pointFormat: '{series.name}: <b>{point.y:.2f}</b>' };
      this.graphOptions.plotOptions = {
        series: {
          stacking: 'normal'
        }
      }

      this.collectionSeries.series.map(function(series){
        // transform arrays from relative value to percentage
        series.non_sus_percentage_arr = series.non_sus_percentage_arr.map(function(x) { return x * 100; });
        series.sus_percentage_arr = series.sus_percentage_arr.map(function(x) { return x * 100; });
        
        this.graphOptions.series.push(
          {
            name: 'Geral '+series.name,
            data: series.non_sus_percentage_arr,
            pointStart: Date.UTC(
              series.start_date.getFullYear(),
              series.start_date.getMonth(),
              series.start_date.getDate()
            ),
            pointInterval: interval
          },
          {
            name: "SUS "+series.name,
            data: series.sus_percentage_arr,
            pointStart: Date.UTC(
              series.start_date.getFullYear(),
              series.start_date.getMonth(),
              series.start_date.getDate()
            ),
            pointInterval: interval
          }
        );
      }, this);
    }
    
    else if (this.selectedGraphType == "linear") {
      this.graphStartDate = new Date(this.collectionSeries.start_date);
      this.graphStartDate.setDate(this.graphStartDate.getDate() + 1);
      let interval: number = 24 * 3600 * 1000; // Graph tick interval in milliseconds
      this.graphOptions.chart = { type: 'line' };
      if (this.selectedInterval == "Mensal" || this.selectedInterval == "Anual")
        interval *= 30;
      if (this.selectedInterval == "Anual")
        interval *= 12;

      this.graphOptions.title = { text: "Atendimentos ao longo do tempo, em valores absolutos" };
      this.graphOptions.yAxis = { title: { text: "Número de pacientes" } };
      this.graphOptions.xAxis = {
        type: "datetime",
        tickInterval: interval
      };


      this.graphOptions.tooltip = { pointFormat: '{series.name}: <b>{point.y:.2f}</b>' };
      this.graphOptions.plotOptions = {
        series: {
          stacking: 'normal'
        },
        column: {
          stacking: 'normal',
          dataLabels: {
            enabled: false,
            formatter: function () {
              return '<span style="text-align:center; color:' + ((Number(this.percentage) < 60) && (this.series.name == 'SUS  - Executadas') ? 'yellow' : 'white') + ';">' + Math.floor(this.percentage * 10)/10 + '%</span>' //this.point.propriedade
            }
            //{point.percentage:.2f}%</span>'
          }
        }
      }
      //note: default dates are objects, input dates are strings
      if ((new Date(this.endDate)).getTime() - (new Date(this.startDate)).getTime() < 604800001) {//menos de 7 dias
        this.graphOptions.plotOptions.column.dataLabels.enabled = true;
      }
      else {
        this.graphOptions.plotOptions.column.dataLabels.enabled = false;
      }

      
      let total_executed: number = 0;
      let n_sus_ex_percentage_arr: any = [];
      let sus_ex_percentage_arr: any = [];
      Array.from(Array(this.collectionSeries.series[0].non_sus_percentage_arr.length).keys()).forEach(index => {
        total_executed = this.collectionSeries.series[0].sus_arr[index] +                        this.collectionSeries.series[0].non_sus_arr[index];
        
        n_sus_ex_percentage_arr[index] = this.collectionSeries.series[0].non_sus_arr[index]/total_executed;
        
        sus_ex_percentage_arr[index] = this.collectionSeries.series[0].sus_arr[index]/total_executed;

      });


      this.graphOptions.series.push(
        {
          name: 'Geral '+this.collectionSeries.series[0].name,
          data: this.collectionSeries.series[0].non_sus_arr,
          pointStart: Date.UTC(
            this.collectionSeries.series[0].start_date.getFullYear(),
            this.collectionSeries.series[0].start_date.getMonth(),
            this.collectionSeries.series[0].start_date.getDate()
          ),
          pointInterval: interval
        },
        {
          name: "SUS "+this.collectionSeries.series[0].name,
          data: this.collectionSeries.series[0].sus_arr,
          pointStart: Date.UTC(
            this.collectionSeries.series[0].start_date.getFullYear(),
            this.collectionSeries.series[0].start_date.getMonth(),
            this.collectionSeries.series[0].start_date.getDate()
          ),
          pointInterval: interval
        }
      );
    }

    else if (this.selectedGraphType == "pie") {
      this.graphOptions.chart = { type: 'pie' };
      this.graphOptions.title = { text: 'Proporção de atendidos pelo SUS' };
      this.graphOptions.tooltip = { pointFormat: '{series.name}: <b>{point.y}</b>' }
      this.graphOptions.plotOptions = {
        pie: {
          dataLabels: {
            enabled: true,
            format: '<b>{point.name}</b>: {point.percentage:.2f}%'
          }
        }
      }
      let data = []
      this.collectionSeries.series.map(function(series){
        data.push(
          {
            name: 'Geral '+series.name,
            y: series.non_sus_total
          },
          {
            name: 'SUS '+series.name,
            y: series.sus_total
          }
        );
      },this)
      this.graphOptions.series = [{
        name: 'Total de atendidos:',
        data: data
      }]
    }

    else if (this.selectedGraphType == "absolute") {
      this.graphStartDate = new Date(this.collectionSeries.start_date);
      this.graphStartDate.setDate(this.graphStartDate.getDate() + 1);

      let interval: number = 24 * 3600 * 1000; // Graph tick interval in milliseconds
      this.graphOptions.chart = { type: 'column' };
      if (this.selectedInterval == "Mensal" || this.selectedInterval == "Anual")
        interval *= 30;
      if (this.selectedInterval == "Anual")
        interval *= 12;

      this.graphOptions.title = { text: "Atendimentos ao longo do tempo, em valores absolutos" };
      this.graphOptions.yAxis = { title: { text: "Número de pacientes" } };
      this.graphOptions.xAxis = {
        type: "datetime",
        tickInterval: interval
      };


      this.graphOptions.tooltip = { pointFormat: '{series.name}: <b>{point.y:.2f}</b>' };
      this.graphOptions.plotOptions = {
        series: {
          stacking: 'normal'
        },
        column: {
          stacking: 'normal',
          dataLabels: {
            enabled: false,
            formatter: function () {
              if (this.series.name ==  'SUS  - Executadas')
                console.log(this.series.name);
              return '<span style="text-align:center; color:' + ((Number(this.percentage) < 60) && (this.series.name == 'SUS  - Executadas') ? 'yellow' : 'white') + ';">' + Math.floor(this.percentage * 10)/10 + '%</span>' 
            }
            //{point.percentage:.2f}%</span>'
          }
        }
      }
      //note: default dates are objects, input dates are strings
      if ((new Date(this.endDate)).getTime() - (new Date(this.startDate)).getTime() < 604800001) {//menos de 7 dias
        this.graphOptions.plotOptions.column.dataLabels.enabled = true;
      }
      else {
        this.graphOptions.plotOptions.column.dataLabels.enabled = false;
      }

      
      let total_executed: number = 0;
      let n_sus_ex_percentage_arr: any = [];
      let sus_ex_percentage_arr: any = [];
      Array.from(Array(this.collectionSeries.series[0].non_sus_percentage_arr.length).keys()).forEach(index => {
        total_executed = this.collectionSeries.series[0].sus_arr[index] +                        this.collectionSeries.series[0].non_sus_arr[index];
        
        n_sus_ex_percentage_arr[index] = this.collectionSeries.series[0].non_sus_arr[index]/total_executed;
        
        sus_ex_percentage_arr[index] = this.collectionSeries.series[0].sus_arr[index]/total_executed;

      });

      //change color of a stack to red if it's below 60% (sus patients only),
      //or do it if non sus patients correspond to 100% of the day's registries:
      //data format (before/after):
      //[ 0, 45, 45, ... ]
      //[ {y: 0, color:"#ffffff"}, {y: 45, color:"#000000"}]
      let sus_arr_withSettings: any = [];
      let non_sus_arr_withSettings: any = [];

      for (var i in this.collectionSeries.series[0].sus_arr) {
          sus_arr_withSettings[i] = {y: this.collectionSeries.series[0].sus_arr[i], color:(sus_ex_percentage_arr[i] < 0.6 ? "#ff0000" : undefined)};
          if (sus_ex_percentage_arr[i] == 0){
              non_sus_arr_withSettings[i] = {y: this.collectionSeries.series[0].non_sus_arr[i], color: '#ff0000'};
          }
          else {
              non_sus_arr_withSettings[i] = sus_ex_percentage_arr[i]
          }
      }

      this.graphOptions.series.push(
        {
          name: 'Geral '+this.collectionSeries.series[0].name,
          data: non_sus_arr_withSettings,
          pointStart: Date.UTC(
            this.collectionSeries.series[0].start_date.getFullYear(),
            this.collectionSeries.series[0].start_date.getMonth(),
            this.collectionSeries.series[0].start_date.getDate()
          ),
          pointInterval: interval
        },
        {
          name: "SUS "+this.collectionSeries.series[0].name,
          data: sus_arr_withSettings,
          pointStart: Date.UTC(
            this.collectionSeries.series[0].start_date.getFullYear(),
            this.collectionSeries.series[0].start_date.getMonth(),
            this.collectionSeries.series[0].start_date.getDate()
          ),
          pointInterval: interval
        }
      );
    }

    else if (this.selectedGraphType == "proportional") {

      this.graphStartDate = new Date(this.collectionSeries.start_date);
      this.graphStartDate.setDate(this.graphStartDate.getDate() + 1);

      let interval: number = 24 * 3600 * 1000; // Graph tick interval in milliseconds
      this.graphOptions.chart = { type: 'column' };
      if (this.selectedInterval == "Mensal" || this.selectedInterval == "Anual")
        interval *= 30;
      if (this.selectedInterval == "Anual")
        interval *= 12;

      this.graphOptions.title = { text: "Atendimentos ao longo do tempo, em valores percentuais" };
      this.graphOptions.yAxis = { 
        title: { text: "Proporção de pacientes" }, 
        tickInterval: 25,
        max: 100
      };
      this.graphOptions.xAxis = {
        type: "datetime",
        tickInterval: interval
      };


      this.graphOptions.tooltip = { pointFormat: '{series.name}: <b>{point.y:.2f}</b>' };
      this.graphOptions.plotOptions = {
        series: {
          stacking: 'normal'
        }
      }
      let total_executed: number = 0;
      let n_sus_ex_percentage_arr: any = [];
      let sus_ex_percentage_arr: any = [];
      Array.from(Array(this.collectionSeries.series[0].non_sus_percentage_arr.length).keys()).forEach(index => {
        
        total_executed = this.collectionSeries.series[0].sus_arr[index] +                        this.collectionSeries.series[0].non_sus_arr[index];
        
        n_sus_ex_percentage_arr[index] = 100*this.collectionSeries.series[0].non_sus_arr[index]/total_executed;
        
        sus_ex_percentage_arr[index] = 100*this.collectionSeries.series[0].sus_arr[index]/total_executed;

      });
    
      this.graphOptions.series.push(
        {
          name: 'Geral '+this.collectionSeries.series[0].name,
          data: n_sus_ex_percentage_arr,
          pointStart: Date.UTC(
            this.collectionSeries.series[0].start_date.getFullYear(),
            this.collectionSeries.series[0].start_date.getMonth(),
            this.collectionSeries.series[0].start_date.getDate()
          ),
          pointInterval: interval
        },
        {
          name: "SUS "+this.collectionSeries.series[0].name,
          data: sus_ex_percentage_arr,
          pointStart: Date.UTC(
            this.collectionSeries.series[0].start_date.getFullYear(),
            this.collectionSeries.series[0].start_date.getMonth(),
            this.collectionSeries.series[0].start_date.getDate()
          ),
          pointInterval: interval
        }
      );
    }
  }


  setTypeofGraph(newType: string) {
    this.selectedGraphType = newType;
  }
  setTypeofStatus(newType: string) {
    this.selectedStatusType = newType;
  }

  public updateChart() {
    /* Retrieve graph data from service */
    this.reportService.getReports(this.selectedInterval, "executed", this.startDate, this.endDate)
      .subscribe(res => {
        this.collectionSeries.clean()
        this.collectionSeries = new CollectionGraphSeries();

        //Subtract the executed uses from all uses and store in this.series
          this.reportService.getReports(this.selectedInterval, "all", this.startDate, this.endDate).subscribe(
              res2 =>{
                  let executed_series: graphSeries = new graphSeries(res, " - Executadas");
                  let not_executed_series: graphSeries = new graphSeries(res2, " - Não executadas");
                  let all_executed_series: graphSeries = new graphSeries(res2, "");

                  for(let i in all_executed_series.sus_arr){
                    not_executed_series.sus_arr[i] = all_executed_series.sus_arr[i] - executed_series.sus_arr[i];
                    not_executed_series.non_sus_arr[i] = all_executed_series.non_sus_arr[i] - executed_series.non_sus_arr[i];
                  }

                  switch(this.selectedStatusType){
                    case "not_executed":
                      not_executed_series.reloadPercentage();
                      this.collectionSeries.clean(not_executed_series);
                      break;
                    case "executed":
                      this.collectionSeries.clean(executed_series);
                      break;
                    case "all":
                      this.collectionSeries.clean(executed_series);
                      this.collectionSeries.add(not_executed_series);
                      this.collectionSeries.normalizePercentage();
                      break;
                  }
              },
              error => console.log(error),
              () => {
                  this.setChartOptions(this.selectedGraphType);
                  this.graphComponent.reset(this.graphOptions);
              }
          );

      },
        error => console.log(error)
      );
  }

}

class CollectionGraphSeries{
  public start_date: string;

  //The default is an empty serie
  public series: Array<graphSeries>;

  constructor(series: Array<graphSeries> = [new graphSeries()]) {
    this.series = series;
  }

  public add(serie: graphSeries){
    this.series.push(serie);
  }

  public clean(newSerie:graphSeries = new graphSeries()){
    this.series = [newSerie];
  }

  public normalizePercentage(){
    let total : Array<number> = [];
    this.series[0].sus_arr.map(function(x,i){

      let sum = this.series.reduce(function(a,b){
        return a+b.sus_arr[i];
      },0);
      let sum2 = this.series.reduce(function(a,b){
        return a+b.non_sus_arr[i];
      },0);

      sum+=sum2

      total.push(sum);
    }, this);

    this.series.map(function(x){
      x.reloadPercentage(total);
    });

  }
}
